<?php
//          (F)
// CMS3 - A Three Content Management System.
// Copyright (C) 2007  Jop... (Jonas F. Jensen).
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

/**
*This file defines and executes the CMS3_MainClass
*
* @package	System
* @author	Jonas F. Jensen <jopsen@gmail.com>
* @copyright	2007 Jonas F. Jensen.
* @license	http://www.gnu.org/licenses/gpl.txt
*/

//Start session
session_start();

/**
*The CMS3 main class, this class hostes all plugins and initiate everything.
*/
class CMS3_MainClass
{
	protected $Plugins = array();

	/**
	* Constructor, loads all plugins
	*/
	function __construct(){
		//Include all shared interfaces
		$InShare = scandir("./share/");
		foreach($InShare as $Shared){
			if($Shared != "." && $Shared != ".."){
				$InShared = scandir("./share/" . $Shared . "/"); //$InShare
				foreach($InShared as $SharedInterface){
					if($SharedInterface != "." && $SharedInterface != ".."){
						//Use output control to avoid problems from bad plugins
						ob_start();
						require_once($this->GetCMS3Path() . "share/". $Shared ."/" . $SharedInterface);
						ob_end_clean();
					}
				}
			}
		}

		$InLib = scandir("./lib/");
		foreach($InLib as $Plugin){
			//Avoid trying to load .. and .
			if($Plugin != "." && $Plugin != ".."){
				//You could do some testing of is_dir etc. but DONT
				$this->Plugins[$Plugin] = $this->LoadPlugin($Plugin);
			}
		}
	}

	/**
	* Destructor, closes Database connections
	*/
	function __destruct() {
		if(isset($this->UserSpaceDatabaseHandle)){
			if(!mysql_close($this->UserSpaceDatabaseHandle)){
				$this->LogMySQL("System", "Error closing userspace MySQL connection",$this->UserSpaceDatabaseHandle);
			}
		}
		if(isset($this->SystemDatabaseHandle)){
			if(!mysql_close($this->SystemDatabaseHandle)){
				$this->LogMySQL("System", "Error closing system MySQL connection",$this->SystemDatabaseHandle);
			}
		}
	}

	/**
	* Main functions start the application
	*/
	static function Main($Args){
		$MainClass = new CMS3_MainClass();
		$MainClass->ResolveIdentifier($Args);
	}

	/**
	 * Gets the username of the current user, resturns null if user not loggedin.
	 *
	 * @return string
	 * @access public
	 */
	public function GetUserName( ) {
		if($_SESSION["loggedin"] == "TruE"){
			return $_SESSION["UserName"];
		}else{
			return false;
		}
	} // end of member function GetUserName

	/**
	*Log the user in
	*
	*@param string Username Username of the user
	*@param string Password The users password
	*@return bool true if login was successfull
	*/
	public function Login($Username, $Password){
		//Calculate hash
		$Password = sha1($Password);
		$Query = "Select BackendAccess from CMS3_Users where Username = '$Username' AND Password = '$Password' limit 1";
		$Result = mysql_query($Query, $this->GetSystemDatabaseHandle());
		if(mysql_num_rows($Result) != 0){
			$_SESSION["loggedin"] = "TruE";
			$_SESSION["UserName"] = $Username;
			//TODO cache backend access value, since it's usefull
			return true;
		}else{
			return false;
		}
	}

	/**
	 * Returns true if current user is administrator.
	 *
	 * @return true/false
	 * @access public
	 */
	public function IsAdmin( ) {
		//As of development just return true
		//return true;
		//We are using sha1 since it's not completly broken yet.
		if($UserName = $this->GetUserName()){
			$DB = $this->GetSystemDatabaseHandle();
			$Result = mysql_query("Select BackendAccess from CMS3_Users where Username = '$UserName' limit 1");
			//We don't have to check password that should be done in GetUserName
			//AND Password = '"  . $_COOKIE["CMS3_PasswordHash"] . "' limit 1",);
			if($Result){
				$BackendAccess = mysql_fetch_row($Result);
				if($BackendAccess[0] == "1"){
					return true;
				}else{
					return false;
				}
			}else{
				return false;
			}
		}
	} // end of member function IsAdmin

	/**
	 * Gets the server path of the CMS3 installation.
	 *
	 * @return string
	 * @access public
	 */
	public function GetCMS3Path( ) {
		//As of development
		return $_SERVER["DOCUMENT_ROOT"] . "/";
	} // end of member function GetCMS3Path

	/**
	 * Gets the URL of the CMS3 toplevel namespace.
	 *
	 * @return string
	 * @access public
	 */
	public function GetCMS3URL( ) {
		//As of development
		return "http://" . $_SERVER["HTTP_HOST"] . "/";
	} // end of member function GetCMS3URL

	/**
	* Load a plugin
	*
	* @param string Plugin PluginID of the plugin you wish to load
	* @return object Returns the plugin you requested
	*/
	public function &LoadPlugin($Plugin){
		//Note this could also be done with autoload, but this will cause less conflicts

		//Use output control to avoid problems from bad plugins
		ob_start();
		require_once($this->GetCMS3Path() . "lib/$Plugin/CMS3_$Plugin.php");
		ob_end_clean();
		$IPlugin = eval("return new CMS3_$Plugin();");
		$IPlugin->Register($this);
		return $IPlugin;
	}

	/**
	* Gets an instance of a plugin, this returns the one loaded at initialization
	*
	* @param string Plugin PluginID of the plugin you wish to load
	* @return object The plugin you requested, or false if not there.
	*/
	public function &GetPlugin($Plugin){
		//Determine if plugin exists
		if(array_key_exists($Plugin, $this->Plugins))
		{
			return $this->Plugins[$Plugin];
		}else{
			return false;
		}
	}

	/**
	* Gets all plugins that implements an interface
	*
	*@param string Interface Interface the plugins must implement
	*@return array Array of objects that implements the interface
	*/
	public function &GetImplementations($Interface){
		$Implementations = array();
		foreach($this->Plugins as $Plugin){
			if(in_array($Interface,class_implements($Plugin))){
				$Implementations[] = $Plugin;
			}
		}
		return $Implementations;
	}

	/**
	*Call a method on all plugins that implements an interface
	*
	*@param string Interface Interface the plugins must implement to get this call
	*@param string Method Method to be called
	*@param array Arguments Array of arguments to be passed to the method.
	*@return array Returns an array of return values from the different methods
	*/
	public function &CallImplementations($Interface, $Method, $Arguments = array()){
		$Returns = array();
		$StrArgs = "";
		for($i = 0; $i <= sizeof($Arguments); $i++){
			if(StrArgs == ""){
				$StrArgs .= "\$Arguments['" . $i . "']";
			}else{
				$StrArgs .= ", \$Arguments['" . $i . "']";
			}
		}
		$Implementations = $this->GetImplementations($Interface);
		foreach($Implementations as $Implementation){
			$Returns[] = eval("return " . $Implementation .  "->" . $Method . "($Args)");
		}
		return $Returns;
	}


	/**
	* Registers a namespace, the plugin that owns this MUST implement IProvidesContent
	* 
	* @param string Namespace String representation of the namespace, max 255 characters.
	* @param string PluginID PluginID of the owner plugin, this must implement IProvidesContent
	* @access public
	* @return bool Returns true if registration was successfull.
	*/
	public function RegisterNamespace($Namespace, $PluginID){
		//System is an integrated part of the CMS3 system, special treatment of it, since it's hardlinked into the system.
		if($Namespace == "System"){
			return false;
		}
		//We don't have to check if it's already registered, since it's defines as primary key
		$Query = "INSERT INTO CMS3_Namespaces (Namespace, Owner) VALUES ('$Namespace','$PluginID')";
		return mysql_query($Query, $this->GetSystemDatabaseHandle());
	}

	/**
	* Unregisters a namespace
	* 
	* @param string Namespace String representation of the namespace.
	* @access public
	* @return bool Returns true if unregistration was successfull.
	*/
	public function UnregisterNamespace($Namespace){
		$Query = "DELETE FROM CMS3_Namespaces WHERE Namespace = '$Namespace' limit 1";
		return mysql_query($Query, $this->GetSystemDatabaseHandle());
	}

	/**
	* Get namespace from owner
	* 
	* @param string PluginID PluginID of the owner plugin
	* @access public
	* @return string Namespace of the plugin or false if nothing was registered.
	*/
	public function GetNamespace($PluginID){
		$Query = "SELECT Namespace FROM CMS3_Namespaces  WHERE Owner='$PluginID' limit 1";
		if(!$Result = mysql_query($Query, $this->GetSystemDatabaseHandle())){
			$this->LogMySQL("System", "Could not resolve namespace for $PluginID, mysqlerror",  $this->GetSystemDatabaseHandle());
			return false;
		}else{
			if(mysql_num_rows($Result) != 0){
				$Row = mysql_fetch_row($Result);
				return $Row[0];
			}else{
				return false;
			}
		}
	}

	/**
	* Handle for the logfile, not always loaded
	* 
	* @access protected
	*/
	protected $LogFileHandle;

	/**
	 * Print a message to the logfile, useful for debuggin information.
	 *
	 * Message are both printed to custom logfile in ./data/System/System.log and send to syslogd local0 savedin /var/log/CMS3.log according to installation notes.
	 *
	 * @param string PluginID PluginID of the plugin that wrote this log message.
	 * @param string msg Message to print to the log. Note: Date and time is printed automaticly.
	 * @access public
	 */
	public function Log( $PluginID,  $msg ) {
		//Write to custom logfile
		if(!isset($this->LogFileHandle)){
			$this->LogFileHandle = fopen($this->GetCMS3Path() . "/data/System/System.log","a+");
		}
		fwrite($this->LogFileHandle,"\n" . date(DATE_RFC822)  ." - ". $PluginID . ": " . $msg);

		//Write to syslog
		define_syslog_variables();
		//Open connection to syslogd
		if(openlog("CMS3" , LOG_CONS | LOG_PID , LOG_LOCAL0)){
			if(!syslog(LOG_ERR , "CMS3 - " . $PluginID . " | " . $msg)){
				fwrite($this->LogFileHandle,"\n" . date(DATE_RFC822)  ." - System: Could not send message to syslogd.");
			}
			//Close connection to syslogd:
			closelog();
		}else{
			fwrite($this->LogFileHandle,"\n" . date(DATE_RFC822)  ." - System: Could not connect to syslogd.");
		}
	} // end of member function Log

	/**
	 * Print a message of an mysql error to the logfile, useful for debuggin information.
	 *
	 * @param string PluginID PluginID of the plugin that wrote this log message.
	 * @param string msg Message to print to the log. Note: Date and time is printed automaticly.
	 * @param resource mysql resource that the error accoured on.
	 * @access public
	 */
	public function LogMySQL( $PluginID,  $msg , &$res) {
		$this->Log($PluginID, $msg . ": \n" . mysql_errno($res) . ": " .  mysql_error($res));
	} // end of member function Log

	/**
	* Handle for the userspace database, not always initialized
	* 
	* @access protected
	*/
	protected $UserSpaceDatabaseHandle;

	/**
	* Gets a handle for the userspace database
	* 
	* @access public
	* @return Returns a MySQL database connection.
	*/
	public function &GetDatabaseHandle(){
		if(!isset($this->UserSpaceDatabaseHandle)){
			require_once($this->GetCMS3Path() . "/etc/System/mysql.conf.php");
			$this->UserSpaceDatabaseHandle = mysql_connect(
							CMS3_System_UserSpaceDatabaseServer, 
							CMS3_System_UserSpaceDatabaseUser,
							CMS3_System_UserSpaceDatabasePassword);
			if(!$this->UserSpaceDatabaseHandle)
			{
				$this->LogMySQL("System", "Could not connect to userspace database host, mysqlerror", $this->UserSpaceDatabaseHandle);
				return false;
			}
		}

		//Make sure it always called...
		if(!mysql_select_db(CMS3_System_UserSpaceDatabasePrefix . "CMS3_Userspace", $this->UserSpaceDatabaseHandle)){
			$this->LogMySQL("System", "Could not select to userspace database, mysqlerror", $this->UserSpaceDatabaseHandle);
			return false;
		}	
		return $this->UserSpaceDatabaseHandle;
	}

	/**
	* Closes the application, with an FatalError log, and print the log messsage to user.
	* 
	* @access public
	*/
	public function FatalError($PluginID, $msg = "Fatal Error, closing the application."){
		$this->Log($PluginID, $msg);
		//TODO: Try using outputhandlers print Error: 503
		print "\nFatal Error: \n" . $PluginID . ": " . $msg;
		//Destruct everything we can:
		$this->__destruct();

		//Exit and kill all remaining code.
		exit;
	}

	/**
	* Handle for the system database, not always initialized
	* 
	* @access private
	*/
	private $SystemDatabaseHandle;

	/**
	* Gets a handle for the system database
	* 
	* @access protected
	* @return Returns a MySQL database connection.
	*/
	protected function &GetSystemDatabaseHandle(){
		if(!isset($this->SystemDatabaseHandle)){
			require_once($this->GetCMS3Path() . "/etc/System/mysql.conf.php");
			$this->SystemDatabaseHandle = mysql_connect(
							CMS3_System_SystemDatabaseServer, 
							CMS3_System_SystemDatabaseUser,
							CMS3_System_SystemDatabasePassword);
			if(!$this->SystemDatabaseHandle)
			{
				$this->LogMySQL("System", "Could not connect to system database host, mysqlerror", $this->SystemDatabaseHandle);
				$this->FatalError("System", "Mysql connection problems");
			}		
		}
		//Make sure it always called...
		if(!mysql_select_db(CMS3_System_SystemDatabasePrefix . "CMS3_System",$this->SystemDatabaseHandle)){
			$this->LogMySQL("System", "Could not select to system database, mysqlerror", $this->SystemDatabaseHandle);
			$this->FatalError("System", "Mysql connection problems");
		}
		return $this->SystemDatabaseHandle;
	}

	/**
	 * Resoles an print the content of an identifier.
	 *
	 * @param string Identifier Identifier, including namespace etc.

	 * @return bool True/false depending on success of failure
	 * @access public
	 */
	public function ResolveIdentifier( $Identifier ) {
		//Load system settings and parse the system database handle to System plugin
		$System = $this->GetPlugin("System");
		$System->SetSystemDatabaseHandle($this->GetSystemDatabaseHandle());
		$SysSettings = $System->LoadSettings();

		//If identifier is zero restore default namespace from CMS3_System
		if($Identifier == ""){
			$Identifier = $SysSettings["DefaultNamespace"];
		}

		//Modify identifier
		$data = explode("/", $Identifier, 2);
		$Namespace = $data[0];
		$PluginSpecificData = $data[1];

		//Load outputhandler
		if(!($OutputHandler = $this->GetPlugin($SysSettings["DefaultOutputHandler"])))
		{
			//Select a random OutputHandler
			$OutputHandlers = $this->GetImplementations("IOutputHandler");
			$OutputHandler =& $OutputHandlers[0];	
		}

		//Set identifier
		$OutputHandler->SetIdentifier($Identifier);

		//get owner of the namespace
		$Query = "SELECT Owner from CMS3_Namespaces where Namespace = '" . $Namespace . "' limit 1";
		if($Result = mysql_query($Query, $this->GetSystemDatabaseHandle()))
		{
			if(mysql_num_rows($Result) != 0){
				$Row = mysql_fetch_row($Result);
				$PluginID = $Row[0];
				$IProvidesContent = $this->GetPlugin($PluginID);
				$IProvidesContent->GetPage($PluginSpecificData, $OutputHandler);
				return true;
			}else{
				//If namespace wasn't found
				$OutputHandler->PrintError("Namespace not found.", "404");
				return false;
			}
		}else{
			$OutputHandler->PrintError("Namespace not found.", "404");
			return false;
		}
	}

	/**
	 * Gets an embedment from it unique identifier, returns embedable HTML.
	 *
	 * @param string Identifier Unique identifier.

	 * @return string
	 * @access public
	 */
	public function GetEmbedment( $Identifier ) {
		//$data = explode("/", $Identifier, 2);
		//TODO: make this work, remember classes  are note coded at singleton
		/*$PluginID = $data[0];
		$PluginSpecificData = $data[1];
		require_once("OutputHandlers/CMS3_HTMLOutputHandler.php");
		$IEmbedable = $this->LoadPlugin($PluginID);
		
		return $IEmbedable->GetEmbedments($PluginSpecificData,$OutputHandler);*/

		return $identifier;
	} // end of member function GetEmbedment

}


CMS3_MainClass::Main($_GET["i"])

?>
