<?php
//              (F)
// CMS3 - A Three Content Management System.
// Copyright (C) 2007  Jop... (Jonas F. Jensen).
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

/**
*This file defines CMS3_Help
*
* @package	Help
* @author	Jonas F. Jensen <jopsen@gmail.com>
* @copyright	2007 Jonas F. Jensen.
* @license	http://www.gnu.org/licenses/gpl.txt
*/

/**
*The Help plugin displays help pages exposed with CMS3_Help_IProvidesHelp
*/
class CMS3_Help extends CMS3_System_Plugin implements IProvidesEditor, ISystemCallback
{

		//ISystemCallback implementation

	/**
	 * Handles a callback from ajax application.
	 *
	 * @param string data Data attached to this callback
	 * @return 
	 * @access public
	 */
	public function SystemCallback( $data ){
		if($Content = $this->ResolveIdentifier($data)){
			$Title = str_replace("/", " / ", $data);
			print "<h1>CMS3 / " . $Title . "</h1>" . $Content;
		}else{
			print "The requested data could not be found.";
		}
	}

		//Implementation of ProvidesEditor

	/**
	 * Shows help pages
	 *
	 * @param IEmbedmentOutputHandler EmbedmentOutputHandler Outputhandler the content MUST be parse to.
	* @param string Identifier String at the end of the URL
	 * @return IEmbedmentOutputHandler
	 * @access public
	 */
	public function &ModifyContent($Identifier = null, &$EmbedmentOutputHandler){
		//Load content from files
		$Head = file_get_contents($this->Owner->GetCMSPath . "data/Help/HelpHead.htm");
		$Body = file_get_contents($this->Owner->GetCMSPath . "data/Help/HelpBody.htm");

		//Generate treenodes
		$TreeNodes = "";
		$ProvidesHelp = $this->Owner->GetImplementations("CMS3_Help_IProvidesHelp");
		foreach($ProvidesHelp as $Help){
			$TreeNodes .= $this->ParseTreeNodes($Help->GetHelp());
		}

		//Resolve identifier, if any
		$Content = false;
		if(!is_null($Identifier) && $Identifier != ""){
			if($Content = $this->ResolveIdentifier($Identifier)){
				$Title = str_replace("/", " / ", $Identifier);
				$Content = "<h1>CMS3 / " . $Title . "</h1>" . $Content; 
			}
		}

		//If no identifier resolved, no content is set... just load something default
		if(!$Content){
			$Content  = "<h1>CMS3 / Help</h1><p>Welcome to the CMS3 systems help plugin, this plugin will help you read help-pages provided by other plugins. In ordinary words you can browse system end-user documentation using the TreeView to the left.</p>";
		}

		//Parse dynamic content to template
		$Body = str_replace("TREENODES", $TreeNodes ,$Body);
		$Body = str_replace("CONTENT", $Content ,$Body);

		$EmbedmentOutputHandler->ExtentHeader($Head);
		$EmbedmentOutputHandler->SetBody($Body);
		//TODO resolve Identifier: Title: CMS3 / PluginID / Subject / Subsubject /...
		return $EmbedmentOutputHandler;
	}

	/**
	* Gets an absolute address of an icon for this plugin
	*/
	public function GetSystemIcon(){
		return $this->Owner->GetCMS3URL() . "bin/Help/help-browser.png";
	}

	/**
	* Gets menu entry string
	*/
	public function GetSystemMenuEntry(){
		return "Help";
	}

		//Custom protected help function

	/**
	*Parse a list of entries into HTML TreeNodes
	*
	*@param array Entries List of entries to parse
	*@param string Parent Identifier used by ResolveIdentifier
	*@return string HTML/dojo representation of TreeNodes
	*/
	protected function ParseTreeNodes($Entries, $Parent = ""){
		$Output = "";
		foreach($Entries as $Entry){
			$Output .= "<div dojoType='TreeNode' title='" . $Entry[0] . "' object='" . $this->Owner->GetCMS3URL() . "System/Callback/Help" . $Parent . "/" . $Entry[0] . "'>";
			$Output .= $this->ParseTreeNodes($Entry[2], $Parent . "/" . $Entry[0]);
			$Output .= "</div>";
		}
		return $Output;
	}

	/**
	*Gets the content of a help page from its identifier
	*
	*@param string Identifier for the information you wish to retrive
	*@param array Entries List of entries to search, defaults all help pages available
	*@return string Content related the identifier, returns false if nothing was found within the searched entries
	*/
	protected function ResolveIdentifier($Identifier, $Entries = null){
		//If not set, get all entries available
		if(is_null($Entries)){
			$Entries = array();
			$ProvidesHelp = $this->Owner->GetImplementations("CMS3_Help_IProvidesHelp");
			//Foreach plugin that provides help
			foreach($ProvidesHelp as $Help){
				//Get Top Level Array
				$TLA = $Help->GetHelp();
				foreach($TLA as $TLEntry){
					//Join top level arrays to one
					$Entries[] = $TLEntry;
				}
			}
		}

		foreach($Entries as $Entry){
			//If the entry is what we're looking for
			if($Entry[0] == $Identifier){
				return $Entry[1];
			}

			//Check if it's owner of the one we're looking for
			$IArray = explode("/", $Identifier, 2);
			if($Entry[0] == $IArray[0]){
				if($Content = $this->ResolveIdentifier($IArray[1], $Entry[2])){
					return $Content;
				}
			}
		}
		//If not already returned return false
		return false;
	}

		//Implementation of IPlugin
	/**
	 * Gets the pluginID of the plugin.
	 *
	 * @return string
	 * @access public
	 */
	public function GetPluginID( ){
		return "Help";
	}


}



?>
