<?php     
//          (F)
// CMS3 - A Three Content Management System.
// Copyright (C) 2007  Jop... (Jonas F. Jensen).
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

/**
*This file defines CMS3_PackageManager
*
* @package	System
* @author	Jonas F. Jensen <jopsen@gmail.com>
* @copyright	2007 Jonas F. Jensen.
* @license	http://www.gnu.org/licenses/gpl.txt
*/

/**
 *PackageManager provides the logic for installation and removal of packages.
 *
 *This plugin doesn't provide any user interface only API interface for package operations.
 */
class CMS3_PackageManager extends CMS3_System_Plugin
{
		//Exposed API

	/**
	*Verifies and installs a package
	*/
	public function InstallPackage($PluginID)
	{
		//TODO detect some of the worst errors and log them

		//Check to see if the package exists
		$PackagePath = $this->Owner->GetCMS3Path() . "data/PackageManager/packages/" . $PluginID . ".c3p";
		if(file_exists($PackagePath)){

			//make sure theres a temp dir
			if(is_dir("/tmp/C3pkg/")){
				//If our tmp dir exists clean it
				shell_exec("rm /tmp/C3pkg/*");
			}else{
				//if not create it.
				shell_exec("mkdir /tmp/C3pkg/");				
			}

			//copy the package
			shell_exec("cp " . $PackagePath . " /tmp/" . $PluginID . ".c3p");			

			//Lets extract the package:
			shell_exec("cd /tmp/C3pkg/; tar -xvf ../" . $PluginID . ".c3p");

			//Lets extract the data from package
			$InstalledFiles = shell_exec("cd " . $this->Owner->GetCMS3Path() . "; tar -xvf /tmp/C3pkg/data.tar");


			//Load all of the shared files of the plugin
			if(file_exists($this->Owner->GetCMS3Path() . "share/" . $PluginID . "/"))
			{
				$Shared = scandir($this->Owner->GetCMS3Path() . "share/" . $PluginID . "/");
				foreach($Shared as $SharedInterface){
					if($SharedInterface != "." && $SharedInterface != ".."){
						//Use output control to avoid problems from bad plugins
						ob_start();
						require_once($this->GetCMS3Path() . "share/". $PluginID ."/" . $SharedInterface);
						ob_end_clean();
					}
				}
			}

			//Check if the plugin implements IInstall
			$Plugin = $this->Owner->LoadPlugin($PluginID);
			$IResult = "";
			if(in_array("IInstall",class_implements($Plugin))){
				$IResult = $Plugin->Install() ? "The installations script was succesfully executed.\n" : "There was error during execution of the installation script, please take a look at the system log.\n";
			}

			return $IResult . $InstalledFiles;
		}else{
			//TODO fix this when implementing remote repositories
			return "Package not found.";
		}
	}

	/**
	*Verifies the integrity of a (.c3p) package, not installed plugin
	*
	*@param string PluginID PluginID of the package to be verify
	*@return bool True if signature is correct.
	*/
	public function VerifyPackage($PluginID)
	{
		//TODO detect some of the worst errors and log them
		//Check to see if the package exists
		$PackagePath = $this->Owner->GetCMS3Path() . "data/PackageManager/packages/" . $PluginID . ".c3p";
		if(file_exists($PackagePath)){

			//make sure theres a temp dir
			if(is_dir("/tmp/C3pkg/")){
				//If our tmp dir exists clean it
				shell_exec("rm /tmp/C3pkg/*");
			}else{
				//if not create it.
				shell_exec("mkdir /tmp/C3pkg/");				
			}

			//copy the package
			shell_exec("cp " . $PackagePath . " /tmp/" . $PluginID . ".c3p");			

			//Lets extract the package:
			shell_exec("cd /tmp/C3pkg/; tar -xvf ../" . $PluginID . ".c3p");

			//Lets verify the signature
			system("gpg --verify /tmp/C3pkg/data.tar.sig", $out); //Out is an output variable
			
			//We have to use a little more direct system commands here, since gpg isn't very happy about the this kind of interaction
			//It would be better to bind into a gpg library


			/*
			It seams gpg won't give you the commandline result text result, which mean that we have to rely on the program return.
			The message printed in commandline ends up in php's log file, this could probably be done more nicely, perhaps detect how signed it
			But for now it's okay that we just receive a boolean result... More advanced features would probably require a C wrapper application...
			*/

			//return the result
			return $out == 0;
			//TODO interact more with GPG, but for now this is find, since complete GPG, signature import etc. is a big task
		}else{
			//TODO fix this when implementing remote repositories
			return false;
		}
	}

	/**
	*Verify the integrity of an installed plugin
	*
	*@param string PluginID Id of the plugin you wish to verify
	*@return string Returns a sha1sum result, or false if master signature was invalid.
	*/
	public function VerifyPlugin($PluginID)	
	{
		//Check to see if the plugin is installed
		$CkPath = $this->Owner->GetCMS3Path() . "data/PackageManager/checksums/" . $PluginID;
		$SigPath = $this->Owner->GetCMS3Path() . "data/PackageManager/checksums/" . $PluginID .  ".sig";
		if(file_exists($CkPath) && file_exists($SigPath)){
			system("gpg --verify " . $SigPath, $out); //Out is an output variable
			//TODO analyze output from sha1sum
			return $out == 0 ? shell_exec("cd " . $this->Owner->GetCMS3Path() . "; sha1sum --check " . $CkPath) : false;
		}else{
			return "Plugin not found.";
		}
	}

	/**
	*Remove an installed plugin
	*
	*Note: if this one returns false, you can't be sure that haft of the package is removed or system perhaps broken.
	*
	*@param string PluginID PluginID of the package to remove
	*@return bool True if package removed correctly
	*/
	public function RemovePlugin($PluginID)
	{
		//Get paths of PackageManager files:
		$CkPath = $this->Owner->GetCMS3Path() . "data/PackageManager/checksums/" . $PluginID;
		$SigPath = $this->Owner->GetCMS3Path() . "data/PackageManager/checksums/" . $PluginID . ".sig";

		if(file_exists($CkPath))
		{
			//Test if the plugin implements IRemove
			$Plugin = $this->Owner->GetPlugin($PluginID);
			if(in_array("IRemove",class_implements($Plugin))){
				$Plugin->Remove();
			}

			/*
			this part of the PackageManager isn't very strong, it can only handle packages that comply with standarts
			*/


			//execute the deletion of the plugin
			$cmd = "cd " . $this->Owner->GetCMS3Path() . "; rm -R */" . $PluginID . "/ " . $CkPath . " " . $SigPath;
			system($cmd , $out);

			//Log any errors, system might be broke on next http request
			if($out != 0)
				$this->Owner->Log($this->GetPluginID(), "Could not remove plugin:" . $PluginID . " error while removing files with following command:\n" . $cmd);

			//Return true/false
			return $out == 0;
		}else{
			//Log an return false
			$this->Owner->Log($this->GetPluginID(), "Could not remove plugin:" . $PluginID . " it wasn't installed by package manager.");
			return false;
		}
	}

	/**
	*Upload a third party package to local repository
	*/
	public function UploadPackage($PackagePath, $PackageName)
	{
		//Set destination
		$Dest = $this->Owner->GetCMS3Path() . "data/PackageManager/packages/" . $PackageName . ".c3p";
		//Check if the file is uploaded or stored otherwise
		if(is_uploaded_file($PackagePath)){
			return move_uploaded_file($PackagePath, $Dest);
		}else{
			return copy($PackagePath, $Dest);
		}
	}

	/**
	*List all the installed plugins that can be deleted or verified
	*/
	public function ListPlugins()
	{
		$Plugins = array();
		$InChecksum = scandir($this->Owner->GetCMS3Path() . "data/PackageManager/checksums/");
		foreach($InChecksum as $Checksum){
			if($Checksum != "." && $Checksum != ".." && substr($Checksum, - 4 , 4) != ".sig"){
				$Plugins[] = $Checksum;
			}
		}
		return $Plugins;
	}

	/**
	*List all the packages that are available for installation in local package repository
	*/
	public function ListPackages()
	{
		//Don't list plugins already installed
		$InstalledPlugins = $this->ListPlugins();
		$Packages = array();
		$InChecksum = scandir($this->Owner->GetCMS3Path() . "data/PackageManager/packages/");
		foreach($InChecksum as $Checksum){
			if($Checksum != "." && $Checksum != ".." && !in_array($Checksum,$InstalledPlugins)){
				$Packages[] = substr($Checksum, 0 , -4);
			}
		}
		return $Packages;
	}


		//Implementation of IPlugin

	/**
	 * Gets the pluginID of the plugin.
	 *
	 * @return string
	 * @access public
	 */
	public function GetPluginID( )
	{
		return "PackageManager";
	}

} // end of CMS3_PackageManager
?>
