<?php     
//          (F)
// CMS3 - A Three Content Management System.
// Copyright (C) 2007  Jop... (Jonas F. Jensen).
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

/**
*This file defines CMS3_StandartOutputHandler
*
* @package	StandartOutputHandler
* @author	Jonas F. Jensen <jopsen@gmail.com>
* @copyright	2007 Jonas F. Jensen.
* @license	http://www.gnu.org/licenses/gpl.txt
*/

/**
 * class CMS3_StandartOutputHandler, a very simple OutputHandler; no configuration available only static template and sitemap.
 */
class CMS3_StandartOutputHandler extends CMS3_StandartOutputHandler_OutputHandler
{

	/** Aggregations: */

	/** Compositions: */

	 /*** Attributes: ***/

	protected $IsDisabled = false;
	protected $Body = "";
	protected $BodyList = null;
	protected $Header = "";
	protected $Title = "Untitled";
	protected $Owner = null;
	protected $Identifier = "";

	/**
	*Sets the complete unique identifier.
	*
	*@param string Idenfitifer Complete unique identifier.
	*@access public
	*/
	public function SetIdentifier($Identifier = ""){
		$this->Identifier = $Identifier;
	}

	/**
	 *
	 * @param CMS3_System Owner The CMS3_System that owns this plugin, gives the plugin ability to get data from
CMS3_System

	 * @return 
	 * @access public
	 */
	public function Register(&$Owner )
	{
		//Just save a handle for now
		$this->Owner =& $Owner;
	}

	/**
	 * Gets the pluginID of the plugin.
	 *
	 * @return string
	 * @access public
	 */
	public function GetPluginID( ){
		return "StandartOutputHandler";
	}

	/**
	 * Data provided by use of this method may be left out if the OutputHandler wishes
	 * to print something else the HTML, therefor don't add important information
	 * here.
	 *
	 * @param string line A line you wish to add to the header.

	 * @access public
	 */
	public function ExtentHeader( $line )
	{
		$this->Header .= $line;
	}


	/**
	 *
	 * @param string title 
	 * @access public
	 */
	public function SetTitle( $title ) {
		$this->Title = $title;
	} // end of member function SetTitle

	/**
	 * Set the body if the page, read parameter documentation.
	 *
	 * Embedments may be included in the body, they shall be formatted as follows: [[EMBED|<identifier>|EMBED]], where <identifier> is the identifier returned from the IEmbedable.
	 * 
	 * @param string body Body of the page, or description of a channel.

	 * @param array List Parse a list, use this feature if you wish to parse a list. This will also
	 * enable to OutputHandler to print a feed. This parameter is optional, if set the
	 * first parameter will define the description of the channel, and every body and
	 * title in this array will define an entry.
	 * Array layout:
	 * [body,body,...]
	 * or
	 * [[title,body],[title,body],...]

	 * @return 
	 * @access public
	 */
	public function SetBody( $body,  $List = null ) {
		$this->Body = $body;
		$this->BodyList = $List;
	} // end of member function SetBody

	/**
	 * Print the content, should also be called from GetPage().
	 *
	 * @access public
	 */
	public function Parse() {
		if(!$this->IsDisabled){
			$Content = $this->GetContent(); // Get the content first, since embedments and header extensions from embedments are parsed here as well.

			//Generating sitemap from Ilinkable
			$iLinks = $this->Owner->GetImplementations("ILinkable");
			$SiteMap = "";
			foreach($iLinks as $iLink)
			{
				foreach($iLink->GetLinks() as $LinkEntry)
				{
					$SiteMap .= $this->ParseLinkEntry($LinkEntry);
				}
			}


			$Template = file_get_contents($this->Owner->GetCMS3Path() . "data/" . $this->GetPluginID() . "/template.htm");
			$Template = str_replace("CMS3URL", $this->Owner->GetCMS3URL() ,$Template);
			$Template = str_replace("<!--TITLE-->",$this->Title ,$Template);
			$Template = str_replace("<!--SITEMAP-->",$SiteMap ,$Template);
			$Template = str_replace("<!--HEADEREXT-->",$this->GetHeader() ,$Template);
			$Template = str_replace("<!--CONTENT-->",$Content ,$Template);

			print $Template;
			$this->Disable();
		}
	} // end of member function Parse


	protected function ParseLinkEntry($Entry)
	{
		$Text = "<a href=\"" . $Entry[1] . "\">" . $Entry[0] . "</a><br />\n";
		$IsFirst = true;
		if($Entry[2] != null){
			foreach($Entry[2] as $SubEntry)
			{
				//Do only add this one time
				if($IsFirst)
				{
					$Text .= "<blockquote>\n";
					$IsFirst = false;
				}
				$Text .= $this->ParseLinkEntry($SubEntry);
			}
		}
		//Do only add this if there were any subentries
		if(!$IsFirst)
			$Text .= "</blockquote>";

		return $Text;
	}


	/**
	*Gets content ready to be printed
	*
	*This does not include header, only the body content.
	*
	*@return string Content ready for print
	*/
	public function GetContent(){
		$Content = "<p>" . $this->ParseEmbedments($this->Body) . "</p>";
		if(is_array($this->BodyList)){
			foreach($this->BodyList as $key => $value){
				if(is_array($value)){
					$Content .= "<h2>" . $this->ParseEmbedments($value[0]) . "</h2>";
					$Content .= "<p>" . $this->ParseEmbedments($value[1]) . "</p>";
				}else{
					$Content .= "<p>" . $this->ParseEmbedments($value) . "</p>";
				}
			}
		}
		return $Content;
	}

	/**
	*Gets the header extension
	*
	*@return string Lines to be added to the header
	*/
	public function GetHeader(){
		return $this->Header;
	}


	/**
	 * Diable the OutputHandler, do this if you wish to print binary data.
	 *
	 * @return 
	 * @access public
	 */
	public function Disable( ){
		$this->IsDisabled = true;
	}
	// end of member function Disable

	/**
	 * Gets to original identifier specified as <Namespace>/<PluginID>.<ext>
	 * Do this if you wish to extraxt the extention and parse manually.
	 *
	 * @return string
	 * @access public
	 */
	public function GetIdentifier( ){
		return $this->Identifier;
	}
	 // end of member function GetIdentifier


	/**
	 * Print an error code, if you use this method no other methods will be available.
	 *
	 * This method will print all 4xx and 5xx error codes specified in the RFC 2616, and some of the codes specified in the WebDAV extension.
	 * NOTE: This mehtod postes a soft error code in HTML, this may be improved later on.
	 *
	 * @param string msg The message you wish to print

	 * @param int code The error code you wish to print eg. 404.

	 * @return 
	 * @access public
	 */
	public function PrintError( $msg,  $code ) {
		//Call parent,since the print error implementation is placed there
		parent::PrintError($msg,$code);

		/*
		I know this isn't needed since the parent implement all the logic, so it should be needed...
		*/
	} // end of member function PrintError

	
	/**
	*Finds and replaces all embedments, in a given string
	*
	*Currently not working since support for embeds isn't implemented yet.
	*
	*@param string Content The string you wish to clear of embedments
	*@return string Content where all embedments have been included.
	*/
	protected function ParseEmbedments($Content){
		//TODO: Parse embedments
		/*$Output = "";
		//Only if there's embedments
		if(sizeof($Exploded = explode("[[EMBED|",$Content)) > 0)
		{
			$Top = sizeof($Exploded);
			for($i = 0; $i < $Top; $i += 2){
				$Output .= $Exploded[$i];
				$data = explode("|EMBED]]",$Exploded[$i +1] );
				$EmbedmentsOutputHandler = $this->Owner->GetEmbedment($data[0]);
				$Output .= $EmbedmentsOutputHandler->GetContent();
				$this->ExtentHeader($EmbedmentsOutputHandler->GetHeader());
				$Output .= $data[1];
			}
		}else{*/
			$Output = $Content;
		//}
		return $Output;
	}

} // end of CMS3_HTMLOutputHandler
?>
