<?php
//              (F)
// CMS3 - A Three Content Management System.
// Copyright (C) 2007  Jop... (Jonas F. Jensen).
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

/**
*This file defines CMS3_System
*
* @package	System
* @author	Jonas F. Jensen <jopsen@gmail.com>
* @copyright	2007 Jonas F. Jensen.
* @license	http://www.gnu.org/licenses/gpl.txt
*/

/**
*CMS_System is a system plugin that hosts IProvidesEditor, IConfigure and ISystemCallback
*/
class CMS3_System extends CMS3_System_Plugin implements IProvidesContent, IConfigure, ISystemCallback, IProvidesEditor
{


	//ISystemCallback implementation
	//used for system callbacks like logout...

	/**
	 * Handles a callback from ajax application.
	 *
	 * @param string data Data attached to this callback

	 * @return 
	 * @access public
	 */
	public function SystemCallback( $data ){
			//Perform a logout
		if($data == "Logout"){
			unset($_SESSION["loggedin"]);
			header("Location: " . $this->Owner->GetCMS3URL() . "System/Login");

			//Save default Namespace
		}elseif($data == "SaveDefaultNamespace"){
			$Settings = $this->LoadSettings();
			$Settings["DefaultNamespace"] = $_POST["DefaultNamespace"];
			if($this->SaveSettings($Settings)){
				print "TRUE";
			}else{
				//Save settings log the error for us.
				print "Unable to save settings, try again or see system log.";
			}

			//Save default OutputHandler
		}elseif($data == "SaveDefaultOutputHandler"){
			$Settings = $this->LoadSettings();
			$Settings["DefaultOutputHandler"] = $_POST["DefaultOutputHandler"];
			if($this->SaveSettings($Settings)){
				print "TRUE";
			}else{
				//Save settings log the error for us.
				print "Unable to save settings, try again or see system log.";
			}

			//Don't know what to do, lets just give it a soft 404... 
		}else{
			print "Error: 404, data not found.";
		}
	}


	//IProvidesEditor implementation
	//is used to create an abstraction of a controlcenter, in other word just a place to list all IConfigure implementations

	/**
	 * Gets an Ajax guide for modification of the content.
	*
	* This implementation of the interface is used to list implemnetations of IConfigure
	 *
	 * @param IEmbedmentOutputHandler EmbedmentOutputHandler Outputhandler the content MUST be parse to.
	 * @return IEmbedmentOutputHandler
	 * @access public
	 */
	public function &ModifyContent($Identifier = null, &$EmbedmentOutputHandler){
		$Configurations = $this->Owner->GetImplementations("IConfigure");
		$Content = "<center><br /><br /><div class='ControlCenter'>";
		foreach($Configurations as $Configuration){
			$Icon = $Configuration->GetControlCenterIcon();
			$Menu = $Configuration->GetControlCenterMenuEntry();
			$Link = $this->Owner->GetCMS3URL() . "System/Configure/" . $Configuration->GetPluginID();
			$Content .= "<a href='$Link'><img src='$Icon' border='0' /><br />$Menu</a>";
		}
		$Content .= "</div></center>";
		//TODO content, publish list of IConfigure, don't list system cause it leads here.
		$EmbedmentOutputHandler->SetBody($Content);
		$EmbedmentOutputHandler->ExtentHeader("<style>.ControlCenter a {display: block; float: left;border: 1px solid #37abc8; background-color: #87cdde; margin: 10px; text-align: center; padding: 2px; font-size: 10px; text-decoration: none; color: black; width: 100px; height: 92px; padding-top: 5px;}\n .ControlCenter a:hover {background-color: #afdde9; text-decoration: none; color: black;} .ControlCenter {width: 55%;}</style>");
		return $EmbedmentOutputHandler;
	}

	/**
	* Gets an absolute address of an icon for this plugin
	*/
	public function GetSystemIcon(){
		return $this->Owner->GetCMS3URL() . "bin/System/preferences-desktop.png";
	}

	/**
	* Gets menu entry string
	*/
	public function GetSystemMenuEntry(){
		return "ControlCenter";
	}


	//IConfigure implementation

	/**
	* Gets an absolute address of an icon for this plugin
	*/
	public function GetControlCenterIcon(){
		return $this->Owner->GetCMS3URL() . "bin/System/preferences-desktop.png";
	}

	/**
	* Gets menu entry string
	*/
	public function GetControlCenterMenuEntry(){
		//TODO: i18n
		return "System Config";
	}

	/**
	 * Gets a dojo based ajax guide for configuration of the plugin.
	 *
	 * @param IEmbedmentOutputHandler OutputHandler 
	 * @return IEmbedmentOutputHandler
	 * @access public
	 */
	public function &Configure($Identifier = null, &$EmbedmentOutputHandler ){
		//Load templates from files
		$Head = file_get_contents($this->Owner->GetCMS3Path() . "data/" . $this->GetPluginID() . "/ConfigHead.htm");
		$Body = file_get_contents($this->Owner->GetCMS3Path() . "data/" . $this->GetPluginID() . "/ConfigBody.htm");

			//Generate content
		//Load settings
		$Settings = $this->LoadSettings();

		//List all namespaces
		$Query = "SELECT Namespace FROM CMS3_Namespaces ORDER BY Namespace";
		$Result = mysql_query($Query, $this->SystemDatabaseHandle);
		$NSOptions = "";
		if(mysql_num_rows($Result) != 0){
			while($Row = mysql_fetch_array($Result, MYSQL_ASSOC))
			{
				if($Settings["DefaultNamespace"] == $Row["Namespace"]){
					 $Selected = " selected=\"selected\"";
				}else{
					$Selected = "";
				}
				$NSOptions .= "<option value='" . $Row["Namespace"] . "'" . $Selected . ">" . $Row["Namespace"] . "</option>\n";
			}
		}

		//List OutputHandlers
		$OHOptions = "";
		$Selected = "";
		$OutputHandlers = $this->Owner->GetImplementations("IOutputHandler");
		foreach($OutputHandlers as $OutputHandler){
			if($Settings["DefaultOutputHandler"] == $OutputHandler->GetPluginID()){
				 $Selected = " selected=\"selected\"";
			}else{
				$Selected = "";
			}
			$OHOptions .= "<option value='" . $OutputHandler->GetPluginID() . "'" . $Selected . ">" . $OutputHandler->GetPluginID() . "</option>\n";
		}

		//Parse dynamic content
		$Body = str_replace("CMS3URL",$this->Owner->GetCMS3URL() ,$Body);
		$Body = str_replace("NSOPTIONS",$NSOptions ,$Body);
		$Body = str_replace("OHOPTIONS",$OHOptions ,$Body);

		//Parse the output
		$EmbedmentOutputHandler->ExtentHeader($Head);
		$EmbedmentOutputHandler->SetBody($Body);
		return $EmbedmentOutputHandler;
	}

	/**
	 * Print the page of a given identifier.
	 *
	 * @param string identifier Identifier of the requested page.

	 * @param IOutputHandler OutputHandler OutputHandler used the parse the output. You may disable this OutputHandler with
	* OutputHandler->Disable() and print the content manually.

	 * @return 
	 * @access public
	 */
	public function &GetPage( $identifier,  &$OutputHandler ){
		if(!$this->Owner->IsAdmin()){
			if($_POST["AjaxRequest"] == "true"){
				//Repond to Ajax callback
				$OutputHandler->Disable();
				if($this->Owner->Login($_POST["Username"], $_POST["Password"])){
					print $this->Owner->GetCMS3URL() . "System/Welcome";
				}else{
					print "Fail";
				}
			}else{
				//TODO: Make login site, using outputhandler
				$OutputHandler->SetTitle("CMS3 System login");
				$Body = file_get_contents($this->Owner->GetCMS3Path() . "data/System/LoginBody.htm");
				$Body = str_replace("CMS3URL",$this->Owner->GetCMS3URL() ,$Body);
				$OutputHandler->SetBody($Body);
				$Header = file_get_contents($this->Owner->GetCMS3Path() . "data/System/LoginHeader.htm");
				$Header = str_replace("CMS3URL",$this->Owner->GetCMS3URL() ,$Header);
				$OutputHandler->ExtentHeader($Header);
				$OutputHandler->Parse();
			}
		}else{
			//We wont be using this...
			$OutputHandler->Disable();

			$data = explode("/", $identifier, 3);
			if(sizeof($data) == 1){
				$SysOutput = $this->GetSystemOutputHandler();
				//TODO: i18n and create an overview of what can be done here...
				$SysOutput->SetBody("Welcome to CMS3 backend, please use the dock for navigation.");
				$this->ParseSystemOutputHandler($SysOutput);
			}else{
				//Append empty string, if there's only two
				if(sizeof($data) == 2){
					$data[2] = "";
				}
				if($data[0] == "Configure"){
					$SysOutput = $this->GetSystemOutputHandler();
					$Configure = $this->Owner->GetPlugin($data[1]);
					$SysOutput = $Configure->Configure($data[2], $SysOutput);
					$this->ParseSystemOutputHandler($SysOutput);
				}
				if($data[0] == "Edit"){
					$SysOutput = $this->GetSystemOutputHandler();
					$Editor = $this->Owner->GetPlugin($data[1]);
					$SysOutput = $Editor->ModifyContent($data[2], $SysOutput);
					$this->ParseSystemOutputHandler($SysOutput);
				}
				if($data[0] == "Callback"){
					//TODO: Check if interface is implementated
					$ICallback = $this->Owner->GetPlugin($data[1]);
					$ICallback->SystemCallback($data[2]);
				}
			}
		}

		return $OutputHandler;
	}

	/**
	*Gets an instance of SystemOutputHandler
	*/
	protected function &GetSystemOutputHandler(){
		require_once($this->Owner->GetCMS3Path() . "lib/System/CMS3_System_SysOutputHandler.php");
		$SysOutputHandler = new CMS3_System_SysOutputHandler();
		$SysOutputHandler->Register($this->Owner);
		return $SysOutputHandler;
	}

	/**
	*Parse SysOutputHandler to HTML
	*/
	protected function ParseSystemOutputHandler(&$OutputHandler){
		//TODO enable reponse as ajax call, perhaps using hidden http parameter like $_POST["IsAjax"]
		print "<html>";
		print "<head>" . $OutputHandler->GetHeader() . "</head>";
		print "<body>" . $OutputHandler->GetContent() . "</body>";
		print "</html>";
	}

	/**
	*Referance to a system database handle
	*/
	protected $SystemDatabaseHandle = null;

	/**
	 * Gets the pluginID of the plugin.
	 *
	 * @return string
	 * @access public
	 */
	public function GetPluginID( ){
		return "System";
	}

	/**
	*Set the system database handle of this object
	*
	*@param resource Handle System database handle
	*/
	public function SetSystemDatabaseHandle(&$Handle){
		$this->SystemDatabaseHandle =& $Handle;
	}

		//Function for saving and loading settings
	/**
	*Opens the settings file for writting
	*
	* NOTE: this method is public because it's linked to CMS3_MainClass
	*/
	public function &LoadSettings(){
		//We're return be referance for performance issues...
		$SettingsFile = $this->Owner->GetCMS3Path() . "etc/" . $this->GetPluginID() . "/Settings.bin"; //.bin since it a binary serialized string
		if(file_exists($SettingsFile)){
			//Read data from file
			if(($data = file_get_contents($SettingsFile)) === FALSE){
				$this->Owner->Log($this->GetPluginID(), "Could not read date from settings file: ./etc/" . $this->GetPluginID() . "/Settings.bin");
				return array();				
			}
			//Deserialize loaded data
			return unserialize($data);
		}else{
			return array();
		}
	}

	/**
	*Save settings to files...
	*/
	protected function SaveSettings(&$Settings){
		$SettingsFile = $this->Owner->GetCMS3Path() . "etc/" . $this->GetPluginID() . "/Settings.bin"; //.bin since it a binary serialized string
		$SettingsPath = $this->Owner->GetCMS3Path() . "etc/" . $this->GetPluginID() . "/";
		//Create dir if not there
		if(!is_dir($SettingsPath)){
			if(!mkdir($SettingsPath)){
				//Log error if couldn't be created
				$this->Owner->Log($this->GetPluginID(), "Could not create directory for settings: ./etc/" . $this->GetPluginID() . "/");
				return false;
			}
		}
		//Check if file is writeable
		if(file_exists($SettingsFile) && !is_writable($SettingsFile)){
			//Log error if file isn't writeable
			$this->Owner->Log($this->GetPluginID(), "Settings file is not writeable: ./etc/" . $this->GetPluginID() . "/Settings.bin");
			return false;
		}else{
			//Open the file for writing
			if(!$Handle = fopen($SettingsFile, "w")){
				$this->Owner->Log($this->GetPluginID(), "Settings file could not be opened: ./etc/" . $this->GetPluginID() . "/Settings.bin");
				return false;
			}
			//Write data to file
			if(fwrite($Handle,serialize($Settings)) === FALSE){
				$this->Owner->Log($this->GetPluginID(), "Settings could not be written to file: ./etc/" . $this->GetPluginID() . "/Settings.bin");
				return false;
			}
			//Close filestream/handle
			if(!fclose($Handle)){
				$this->Owner->Log($this->GetPluginID(), "Could not close setting file correctly: ./etc/" . $this->GetPluginID() . "/Settings.bin");
			}
			//Return true since we've saved the settings, doesn't really matter if file wasn't closed correctly 
			return true;
		}
	}
}


?>
