<?php
//              (F)
// CMS3 - A Three Content Management System.
// Copyright (C) 2007  Jop... (Jonas F. Jensen).
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

/**
*This file defines CMS3_System_SysOutputHandler
*
* @package	System
* @author	Jonas F. Jensen <jopsen@gmail.com>
* @copyright	2007 Jonas F. Jensen.
* @license	http://www.gnu.org/licenses/gpl.txt
*/

/**
*An IEmbedmentOutputHandler implementation for CMS3_System, internal use only
*/
class CMS3_System_SysOutputHandler implements IEmbedmentOutputHandler
{


	/**
	*Body of the processing page
	*/
	protected $Body = "";

	/**
	*Header of the processing page
	*/
	protected $Header = "";

	/**
	*BodyList of the processing page
	*/
	protected $BodyList = null;


	/**
	 * Data provided by use of this method may be left out if the OutputHandler wishes
	 * to print something else the HTML, therefor don't add important information
	 * here.
	 *
	 * @param string line A line you wish to add to the header.

	 * @return 
	 * @access public
	 */
	public function ExtentHeader( $line ){
		$this->Header .= $line;
	}

	/**
	 * Set the body if the page, read parameter documentation.
	 *
	 * @param string body Body of the page, or description of a channel.

	 * @param array List Parse a list, use this feature if you wish to parse a list. This will also
	 * enable to OutputHandler to print a feed. This parameter is optional, if set the
	 * first parameter will define the description of the channel, and every body and
	 * title in this array will define an entry.
	 * Array layout:
	 * [body,body,...]
	 * or
	 * [[title,body],[title,body],...]
	 * @access public
	 */
	public function SetBody( $body,  $List = null ){
		$this->Body = $body;
		$this->BodyList = $List;
	}
	 // end of member function SetBody


	/**
	*Gets content ready to be printed
	*
	*This does not include header, only the body content.
	*
	*@return string Content ready for print
	*/
	public function GetContent(){
		$Body = file_get_contents($this->Owner->GetCMS3Path() . "data/System/SystemBody.htm");
		$Menu = "";
		$Editors = $this->Owner->GetImplementations("IProvidesEditor");
		foreach($Editors as $Editor){
			$Menu .= "<a dojoType='FisheyeListItem' onClick=\"document.location = 'CMS3URLSystem/Edit/" . $Editor->GetPluginID() . "';\" href='CMS3URLSystem/Edit/" . $Editor->GetPluginID() . "' iconsrc='" . $Editor->GetSystemIcon() . "' title='" . $Editor->GetSystemMenuEntry()  . "' caption='" . $Editor->GetSystemMenuEntry()  . "' ><img border='0' src='" . $Editor->GetSystemIcon() . "' /></a>";
		}
		$Body = str_replace("MENUOPTIONS",$Menu ,$Body);
		$Body = str_replace("CMS3URL",$this->Owner->GetCMS3URL() ,$Body);
		$Body .= "<p>" . $this->Body . "</p>";
		if(is_array($this->BodyList)){
			foreach($this->BodyList as $value){
				if(is_array($value)){
					$Body .= "<b>" . $value[0] . "</b><br>";
					$Body .=  $value[1] . "<br>";
				}else{
					$Body .= $value . "<br>";
				}
			}
		}
		return $Body;
	}

	/**
	*Gets the header extension
	*
	*@return string Lines to be added to the header
	*/
	public function GetHeader(){
		$Header = file_get_contents($this->Owner->GetCMS3Path() . "data/System/SystemHeader.htm");
		$Header = str_replace("CMS3URL",$this->Owner->GetCMS3URL() ,$Header);
		return $Header . $this->Header;
	}


	/**
	*Const defining the PluginID
	*/
	private static $PluginID = "System";

	/**
	*Referance to the MainClass owning this instance
	*/
	protected $Owner = null;

	/**
	 *
	 * @param CMS3_System Owner The CMS3_System that owns this plugin, gives the plugin ability to get data from CMS3_System
	 * @access public
	 */
	public function Register( &$Owner ){
		$this->Owner =& $Owner;
	}

	/**
	 * Gets the pluginID of the plugin.
	 *
	 * @return string
	 * @access public
	 */
	public function GetPluginID( ){
		return $this->PluginID;
	}

}











?>
